package com.llisoft.sms.service;

import java.util.Objects;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.llisoft.sms.config.MtaException;
import com.llisoft.sms.util.CodeUtil;
import com.llisoft.sms.util.JsonUtil;
import com.llisoft.sms.util.RedisUtil;
import com.llisoft.sms.vo.SendCodeRequestVo;

/**
 * 短信服务
 * @author YangJie [2019年4月25日]
 */
@Service
public class SmsService {
	
	private Logger logger = LoggerFactory.getLogger(SmsService.class);
	
	@Value("${mta.debug}")
	private boolean debug;
	
	@Autowired
	private AliyunService aliyunService;
	
	
	/**
	 * 发送验证码
	 * 同手机号 每分钟1条/每小时5条/每天10条
	 * @param phone
	 * @throws Exception 
	 */
	public boolean sendCode(SendCodeRequestVo requestVo, String ip) throws Exception {
		// 检查人机验证码
		logger.info("人机验证: ", JsonUtil.toJson(requestVo));
		if (debug) {
			logger.warn("开发模式：人机验证已跳过");
		}
		if (!debug && !aliyunService.afs(requestVo.getAppkey(), requestVo.getScene(), requestVo.getToken(), requestVo.getSessionid(), requestVo.getSig(), ip)) {
			logger.warn("人机验证失败：{}", requestVo.getToken());
			throw new MtaException("人机验证失败，请刷新页面后重试");
		}
		// 检查号码
		String phone = requestVo.getPhone();
		logger.info("检查手机号: {}", phone);
		if(!this.checkSend(phone)) {
			logger.warn("手机号被限制发送：{}", phone);
			throw new MtaException("此号码被限制发送：" + phone);
		}
		// 发送短信
		String code = String.valueOf(CodeUtil.ints(4));
		RedisUtil.set(phone + "_" + code, "", 10, TimeUnit.MINUTES); // 有效期10分钟
		logger.info("发送验证码: {}", code);
		aliyunService.sms(requestVo.getSign(), phone, code); // 第三方发送短信（异步）
		return true;
	}
	
	/**
	 * 检查发送
	 * @param phone
	 * @return
	 * @throws Exception 
	 */
	private boolean checkSend(String phone) throws Exception {
		// 每分钟1条
		String key = phone + "_limit_minute";
		String result = RedisUtil.get(key);
		if(Objects.nonNull(result)) {
			logger.warn("每分钟限制1条: {}", phone);
			return false;
		}
		RedisUtil.set(key, "", 1, TimeUnit.MINUTES);
		// 每小时5条
		key = phone + "_limit_hour";
		result = RedisUtil.get(key);
		if(Objects.nonNull(result)) {
			int count = Integer.parseInt(result);
			if(count >= 5) {
				logger.warn("每小时限制5条: {}", phone);
				return false;
			}
			RedisUtil.set(key, count+1, 1, TimeUnit.HOURS);
		}else {
			RedisUtil.set(key, 1, 1, TimeUnit.HOURS);
		}
		// 每天10条
		key = phone + "_limit_day";
		result = RedisUtil.get(key);
		if(Objects.nonNull(result)) {
			int count = Integer.parseInt(result);
			if(count >= 10) {
				logger.warn("每天限制10条: {}", phone);
				return false;
			}
			RedisUtil.set(key, count+1, 1, TimeUnit.DAYS);
		}else {
			RedisUtil.set(key, 1, 1, TimeUnit.DAYS);
		}
		return true;
	}

	/**
	 * 检查验证码
	 * @param phone
	 * @param code
	 * @return
	 */
	public boolean checkCode(String phone, String code) {
		logger.info("检查验证码: {} : {}", phone, code);
		if (debug && "123123".equals(code)) {
			logger.warn("开发模式：默认验证码【123123】通过");
			return true;
		}
		String result = RedisUtil.get(phone + "_" + code);
		if (Objects.nonNull(result)) {
			RedisUtil.delete(phone + "_" + code); // 阅后即焚
			return true;
		}
		return false;
	}
	
}
